@@ -69,7 +69,7 @@
 #' \code{"M"} or \code{"MM"}, \code{NULL} if the fitting method is \code{"OLS"}.}
 #' \item{hatvalues}{a matrix of hatvalues. For \code{method = "OLS"}, the matrix consists of two
 #' columns, for each of the stage-1 and stage-2 regression; for \code{method = "M"} or \code{"MM"},
-#' there is one column for \emph{each} stage=1 regression and for the stage-2 regression. }
+#' there is one column for \emph{each} stage-1 regression and for the stage-2 regression. }
 #' @seealso \code{\link{ivreg}}, \code{\link[stats:lmfit]{lm.fit}}, 
 #' \code{\link[stats:lmfit]{lm.wfit}}, \code{\link[MASS]{rlm}}, \code{\link[stats]{mad}}
 #' @keywords regression
@@ -131,7 +131,9 @@ ivreg.fit <- function(x, y, z, weights, offset, method = c("OLS", "M", "MM"),
   exog <- structure(seq_along(colnames(x)), .Names = colnames(x))
   if(!is.null(auxreg)) {
     endo <- which(colMeans(as.matrix(auxreg$residuals^2)) > sqrt(.Machine$double.eps))
-    inst <- which(rowMeans(as.matrix(coef(auxreg)^2)[, -endo, drop = FALSE]) < sqrt(.Machine$double.eps))
+    inst <- rowMeans(as.matrix(coef(auxreg)^2)[, -endo, drop = FALSE])
+    inst <- which(inst < sqrt(.Machine$double.eps) | is.nan(inst))
+    endo <- exog[endo]
     exog <- exog[-endo]
   } else {
     endo <- inst <- integer()
